1,999bd
e Makefile.PL
e lib/Devel/Cover.pm
e lib/Devel/Cover/DB.pm
e lib/Devel/Cover/Op.pm
e Cover.xs
e cover
e cpancover
e gcov2perl
e create_gold
e lib/Devel/Cover/Test.pm
e lib/Devel/Cover/Criterion.pm
e lib/Devel/Cover/Statement.pm
e lib/Devel/Cover/Branch.pm
e lib/Devel/Cover/Condition.pm
e lib/Devel/Cover/Pod.pm
e lib/Devel/Cover/Time.pm
e lib/Devel/Cover/DB/File.pm
e lib/Devel/Cover/Report/Text.pm
e lib/Devel/Cover/Report/Html_subtle.pm
e CHANGES
e TODO
e MANIFEST
e BUGS
e #1
